searchData={"items":[{"type":"module","title":"ExCurl","doc":"Documentation for `ExCurl`.","ref":"ExCurl.html"},{"type":"module","title":"Shared options - ExCurl","doc":"* `:headers` - a map of headers to include in the request, defaults to `%{\"user-agent\" => \"ex_curl/0.3.0\"}`\n  * `:body` - a string to send as the request body, defaults to `\"\"`\n  * `:follow_location` - if redirects should be followed, defaults to `true`\n  * `:ssl_verifyhost` - if SSL certificates should be verified, defaults to `true`\n  * `:ssl_verifypeer` - if SSL certificates should be verified, defaults to `true`\n  * `:return_metrics` - if request timing metrics should be included in the returned results, defaults to `false`\n  * `:verbose` - if curl should output verbose logs to stdout, useful for debugging. Defaults to `false`\n  * `:http_auth_negotiate` - if curl should use HTTP Negotiation (SPNEGO) as defined in [RFC 4559](https://datatracker.ietf.org/doc/html/rfc4559).\n    Note: this flag requires curl to be compiled with a suitable GSS-API or SSPI library. Defaults to `false`","ref":"ExCurl.html#module-shared-options"},{"type":"module","title":"Error messages - ExCurl","doc":"Error messages refer to error codes on the [libcurl error codes documentation page](https://curl.se/libcurl/c/libcurl-errors.html).\n\nFor example, when we try to send a request to an invalid url:\n\n\n      iex> ExCurl.get(\"https://\")\n      {:error, \"URL_MALFORMAT\"}\n\nThe returned error tuple includes the error message `\"URL_MALFORMAT\"`. This corresponds to the `CURLE_URL_MALFORMAT` (error code 3) error listed\nin the [libcurl error codes documentation](https://curl.se/libcurl/c/libcurl-errors.html).","ref":"ExCurl.html#module-error-messages"},{"type":"function","title":"ExCurl.delete/2","doc":"Sends a DELETE request to the given `url`.\n\nReturns either `{:ok, %ExCurl.Response{}}` or `{:error, \"CURL_ERROR_MESSAGE\"}`.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.\n\nSee the [\"Error messages\"](#module-error-messages) section for details on error messages.","ref":"ExCurl.html#delete/2"},{"type":"function","title":"Examples - ExCurl.delete/2","doc":"iex> ExCurl.delete(\"https://httpbin.org/delete\", headers: %{\"Authorization\" => \"bearer secret\"})","ref":"ExCurl.html#delete/2-examples"},{"type":"function","title":"ExCurl.delete!/2","doc":"Sends a DELETE request to the given `url`. Similar to `delete/2` but raises `ExCurl.CurlError` if the request fails.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.","ref":"ExCurl.html#delete!/2"},{"type":"function","title":"Examples - ExCurl.delete!/2","doc":"iex> ExCurl.delete!(\"https://httpbin.org/delete\")","ref":"ExCurl.html#delete!/2-examples"},{"type":"function","title":"ExCurl.get/2","doc":"Sends a GET request to the given `url`.\n\nReturns either `{:ok, %ExCurl.Response{}}` or `{:error, \"CURL_ERROR_MESSAGE\"}`.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.\n\nSee the [\"Error messages\"](#module-error-messages) section for details on error messages.","ref":"ExCurl.html#get/2"},{"type":"function","title":"Examples - ExCurl.get/2","doc":"iex> {:ok, %ExCurl.Response{status_code: status_code}} = ExCurl.get(\"https://google.com\", follow_location: false)\n     iex> status_code\n     301","ref":"ExCurl.html#get/2-examples"},{"type":"function","title":"ExCurl.get!/2","doc":"Sends a GET request to the given `url`. Similar to `get/2` but raises `ExCurl.CurlError` if the request fails.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.","ref":"ExCurl.html#get!/2"},{"type":"function","title":"Examples - ExCurl.get!/2","doc":"iex> %ExCurl.Response{status_code: status_code} = ExCurl.get!(\"https://google.com\")\n    iex> status_code\n    200","ref":"ExCurl.html#get!/2-examples"},{"type":"function","title":"ExCurl.patch/2","doc":"Sends a PATCH request to the given `url`.\n\nReturns either `{:ok, %ExCurl.Response{}}` or `{:error, \"CURL_ERROR_MESSAGE\"}`.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.\n\nSee the [\"Error messages\"](#module-error-messages) section for details on error messages.","ref":"ExCurl.html#patch/2"},{"type":"function","title":"Examples - ExCurl.patch/2","doc":"iex> ExCurl.patch(\"https://httpbin.org/patch\", headers: %{\"user-agent\" => \"custom-user-agent\"})","ref":"ExCurl.html#patch/2-examples"},{"type":"function","title":"ExCurl.patch!/2","doc":"Sends a PATCH request to the given `url`. Similar to `patch/2` but raises `ExCurl.CurlError` if the request fails.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.","ref":"ExCurl.html#patch!/2"},{"type":"function","title":"Examples - ExCurl.patch!/2","doc":"iex> ExCurl.patch!(\"https://httpbin.org/patch\", body: Jason.encode!(%{\"some-value\" => true}), headers: %{\"content-type\" => \"application/json\"})","ref":"ExCurl.html#patch!/2-examples"},{"type":"function","title":"ExCurl.post/2","doc":"Sends a POST request to the given `url`.\n\nReturns either `{:ok, %ExCurl.Response{}}` or `{:error, \"CURL_ERROR_MESSAGE\"}`.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.\n\nSee the [\"Error messages\"](#module-error-messages) section for details on error messages.","ref":"ExCurl.html#post/2"},{"type":"function","title":"Examples - ExCurl.post/2","doc":"iex> {:ok, %ExCurl.Response{body: body}} = ExCurl.post(\"https://httpbin.org/post\", body: \"some-value=true\")\n     iex> Jason.decode!(body)[\"form\"]\n     %{\"some-value\" => \"true\"}","ref":"ExCurl.html#post/2-examples"},{"type":"function","title":"ExCurl.post!/2","doc":"Sends a POST request to the given `url`. Similar to `post/2` but raises `ExCurl.CurlError` if the request fails.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.","ref":"ExCurl.html#post!/2"},{"type":"function","title":"Examples - ExCurl.post!/2","doc":"iex> ExCurl.post!(\"https://httpbin.org/post\", body: \"some-value=true\")","ref":"ExCurl.html#post!/2-examples"},{"type":"function","title":"ExCurl.put/2","doc":"Sends a PUT request to the given `url`.\n\nReturns either `{:ok, %ExCurl.Response{}}` or `{:error, \"CURL_ERROR_MESSAGE\"}`.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.\n\nSee the [\"Error messages\"](#module-error-messages) section for details on error messages.","ref":"ExCurl.html#put/2"},{"type":"function","title":"Examples - ExCurl.put/2","doc":"iex> ExCurl.put(\"https://httpbin.org/put\", body: \"some-value=true\")","ref":"ExCurl.html#put/2-examples"},{"type":"function","title":"ExCurl.put!/2","doc":"Sends a PUT request to the given `url`. Similar to `put/2` but raises `ExCurl.CurlError` if the request fails.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.","ref":"ExCurl.html#put!/2"},{"type":"function","title":"Examples - ExCurl.put!/2","doc":"iex> ExCurl.put!(\"https://httpbin.org/put\", headers: %{\"content-type\" => \"application/json\"}, body: \"{}\")","ref":"ExCurl.html#put!/2-examples"},{"type":"function","title":"ExCurl.request/3","doc":"Send a request to the `url` using the provided `method` and options\n\nReturns either `{:ok, %ExCurl.Response{}}` or `{:error, \"CURL_ERROR_MESSAGE\"}`.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.\n\nSee the [\"Error messages\"](#module-error-messages) section for details on error messages.","ref":"ExCurl.html#request/3"},{"type":"function","title":"Examples - ExCurl.request/3","doc":"iex> ExCurl.request(\"GET\", \"https://google.com\")","ref":"ExCurl.html#request/3-examples"},{"type":"function","title":"ExCurl.request!/3","doc":"Sends a request to the given `url` using the provided `method` and options. Similar to `request/3` but raises `ExCurl.CurlError` if the request fails.\n\n\nSee the [\"Shared options\"](#module-shared-options) section at the module documentation for available options and their defaults.","ref":"ExCurl.html#request!/3"},{"type":"behaviour","title":"ExCurl.Client","doc":"A helper to create HTTP Clients based on ExCurl that can share common options.","ref":"ExCurl.Client.html"},{"type":"behaviour","title":"Example HTTP Client - ExCurl.Client","doc":"You can `use ExCurl.Client` on any module to provide HTTP Client functionality and share default options. Below is an example based on the great \n[Tesla documentation](https://hexdocs.pm/tesla/readme.html):\n\n```elixir\ndefmodule GitHubClient do\n  use ExCurl.Client, defaults: [base_url: \"https://api.github.com\", headers: %{\"Authorization\" => \"Bearer some-secret-token\"}]\n\n  def user_repos(username), do: get(\"/users/#{username}/repos\")\nend\n```\n\nWe can then use this client with the shared defaults and custom functions:\n\n```elixir\n# A custom GET request using our default base_url and headers\nGitHubClient.get(\"/users/danielrudn\")\n\n# Our custom function, requiring only a username\nGitHubClient.user_repos(\"danielrudn\")\n```","ref":"ExCurl.Client.html#module-example-http-client"},{"type":"behaviour","title":"Response Handling - ExCurl.Client","doc":"You may define an optional `handle_response/1` callback to transform a response after a successful request.\nA common example would be parsing a JSON string into a map. Below we have the same `GitHub` client as above\nbut we also parse the JSON response if possible using the `handle_response/1` callback:\n\n```elixir\ndefmodule GitHubClient do\n  use ExCurl.Client, defaults: [base_url: \"https://api.github.com\", headers: %{\"Authorization\" => \"Bearer some-secret-token\"}]\n\n  def handle_response(%ExCurl.Response{body: body}) do\n    case Jason.decode(body) do\n      {:ok, decoded_body} -> decoded_body\n      _ -> body\n    end\n  end\n\n  def user_repos(username), do: get(\"/users/#{username}/repos\")\nend\n```","ref":"ExCurl.Client.html#module-response-handling"},{"type":"behaviour","title":"Options - ExCurl.Client","doc":"All of the [Shared Options](https://hexdocs.pm/ex_curl/ExCurl.html#module-shared-options) from ExCurl are available. There are also the following additional options:\n\n* `base_url` - The base URL used for all requests by this client module","ref":"ExCurl.Client.html#module-options"},{"type":"callback","title":"ExCurl.Client.handle_response/1","doc":"See [Response Handling](#module-response-handling) for usage and examples.","ref":"ExCurl.Client.html#c:handle_response/1"},{"type":"module","title":"ExCurl.CurlErrorCodes","doc":"Helper functions to map [libcurl error codes](https://curl.se/libcurl/c/libcurl-errors.html) from their integer value to the string value and vice versa.","ref":"ExCurl.CurlErrorCodes.html"},{"type":"function","title":"ExCurl.CurlErrorCodes.get_code/1","doc":"Returns the integer code for the corresponding string error message.\n\nReturns `-1` if the message is not a valid curl error code.","ref":"ExCurl.CurlErrorCodes.html#get_code/1"},{"type":"function","title":"Examples - ExCurl.CurlErrorCodes.get_code/1","doc":"iex> ExCurl.CurlErrorCodes.get_code(\"URL_MALFORMAT\")\n    3\n\n    iex> ExCurl.CurlErrorCodes.get_code(\"DOESN'T EXIST\")\n    -1","ref":"ExCurl.CurlErrorCodes.html#get_code/1-examples"},{"type":"function","title":"ExCurl.CurlErrorCodes.get_message/1","doc":"Returns the string message for a coresponding integer error code.\n\nReturns `\"CODE_OUT_OF_RANGE` if the value is not a valid curl error code.","ref":"ExCurl.CurlErrorCodes.html#get_message/1"},{"type":"function","title":"Examples - ExCurl.CurlErrorCodes.get_message/1","doc":"iex> ExCurl.CurlErrorCodes.get_message(3)\n    \"URL_MALFORMAT\"\n\n    iex> ExCurl.CurlErrorCodes.get_message(256)\n    \"CODE_OUT_OF_RANGE\"","ref":"ExCurl.CurlErrorCodes.html#get_message/1-examples"},{"type":"module","title":"ExCurl.Response","doc":"A valid response returned by ExCurl.","ref":"ExCurl.Response.html"},{"type":"module","title":"Metrics - ExCurl.Response","doc":"When the `return_metrics` option is set to `true` for a request, the `metrics` field will be populated with a [`%ExCurl.ResponseMetrics{}`](https://hexdocs.pm/ex_curl/ExCurl.ResponseMetrics.html) value.\n\nFor example:\n\n      iex> ExCurl.get!(\"https://httpbin.org/get\", return_metrics: true)\n      %ExCurl.Response{status_code: 200, metrics: %ExCurl.ResponseMetrics{}, ...}","ref":"ExCurl.Response.html#module-metrics"},{"type":"type","title":"ExCurl.Response.t/0","doc":"","ref":"ExCurl.Response.html#t:t/0"},{"type":"module","title":"ExCurl.ResponseMetrics","doc":"Request timing metrics.","ref":"ExCurl.ResponseMetrics.html"},{"type":"module","title":"Available Fields - ExCurl.ResponseMetrics","doc":"The following metrics are returned when the `return_metrics` option is set to `true`:\n\n  * `total_time` - corresponds to the [CURLINFO_TOTAL_TIME_T option](https://curl.se/libcurl/c/CURLINFO_TOTAL_TIME_T.html).\n  * `namelookup_time` - corresponds to the [CURLINFO_NAMELOOKUP_TIME_T option](https://curl.se/libcurl/c/CURLINFO_NAMELOOKUP_TIME_T.html).\n  * `connect_time` - corresponds to the [CURLINFO_CONNECT_TIME_T option](https://curl.se/libcurl/c/CURLINFO_CONNECT_TIME_T.html).\n  * `appconnect_time` - corresponds to the [CURLINFO_APPCONNECT_TIME_T option](https://curl.se/libcurl/c/CURLINFO_APPCONNECT_TIME_T.html).\n  * `pretransfer_time` - corresponds to the [CURLINFO_PRETRANSFER_TIME_T option](https://curl.se/libcurl/c/CURLINFO_PRETRANSFER_TIME_T.html).\n  * `starttransfer_time` - corresponds to the [CURLINFO_STARTTRANSFER_TIME_T option](https://curl.se/libcurl/c/CURLINFO_STARTTRANSFER_TIME_T.html).","ref":"ExCurl.ResponseMetrics.html#module-available-fields"},{"type":"type","title":"ExCurl.ResponseMetrics.t/0","doc":"","ref":"ExCurl.ResponseMetrics.html#t:t/0"},{"type":"exception","title":"ExCurl.CurlError","doc":"","ref":"ExCurl.CurlError.html"},{"type":"extras","title":"ExCurl","doc":"# ExCurl [![CI](https://github.com/danielrudn/ex_curl/actions/workflows/ci.yml/badge.svg)](https://github.com/danielrudn/ex_curl/actions/workflows/ci.yml) [![Hex.pm](https://img.shields.io/hexpm/v/ex_curl.svg)](https://hex.pm/packages/ex_curl) [![Hexdocs.pm](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ex_curl/)\n\nElixir bindings for libcurl using Zig's C interoperability.","ref":"readme.html"},{"type":"extras","title":"Installation - ExCurl","doc":"The package can be installed by adding `ex_curl` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_curl, \"~> 0.3.0\"}\n  ]\nend\n```\n\nTo install `ex_curl` in a Livebook, you can use `Mix.install/2`:\n\n```elixir\nMix.install([\n  {:ex_curl, \"~> 0.3.0\"}\n])\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ExCurl","doc":"```elixir\nExCurl.get(\"https://httpbin.org/get\")\n# => {:ok, %ExCurl.Response{status_code: 200, headers: %{\"content-type\" => \"application/json\", body: \"{...}\"}}}\n\n# view request metrics\nExCurl.get!(\"https://httpbin.org/get\", return_metrics: true)\n# => %ExCurl.Response{status_code: 200, metrics: %ExCurl.ResponseMetrics{total_time: 0.2, ...}, ...}}\n\n# submit a form\nExCurl.post(\"https://httpbin.org/post\", body: \"text=#{URI.encode_www_form(\"some value\")}\")\n# => {:ok, %ExCurl.Response{status_code: 200, body: \"{...}\"}}}\n```\n\nGeneral usage examples and documentation can be found on the [ExCurl module documentation page](https://hexdocs.pm/ex_curl/ExCurl.html).","ref":"readme.html#usage"},{"type":"extras","title":"HTTP Clients and Shared Defaults - ExCurl","doc":"```elixir\n# create clients and set default options\ndefmodule CustomClient do\n  use ExCurl.Client, defaults: [headers: %{\"User-Agent\" => \"custom-user-agent\"}]\nend\n\n# all requests using CustomClient will use the custom User-Agent header\nCustomClient.get(\"https://httpbin.org/get\")\n# => {:ok, %ExCurl.Response{status_code: 200, body: \"..\"}}\n```\n\nYou can `use ExCurl.Client` on any module to create HTTP Clients.\nBelow is an example of a GitHub API client based on the great [Tesla documentation](https://hexdocs.pm/tesla/readme.html):\n\n```elixir\ndefmodule GitHubClient do\n  use ExCurl.Client, defaults: [base_url: \"https://api.github.com\"]\n\n  def user_repos(username), do: get(\"/users/#{username}/repos\")\nend\n\n# we can now call the GitHubClient with the pre-defined HTTP verb functions:\nGitHubClient.get(\"/users/danielrudn/repos\")\n# => {:ok, %ExCurl.Response{status_code: 200, body: \"{....}\"}}\n\n# or call our custom defined functions:\nGitHubClient.user_repos(\"danielrudn\")\n# => {:ok, %ExCurl.Response{status_code: 200, body: \"{....}\"}}\n```\n\nMore examples and details on using `ExCurl.Client` are available on the [ExCurl.Client module documentation page](https://hexdocs.pm/ex_curl/ExCurl.Client.html).","ref":"readme.html#http-clients-and-shared-defaults"},{"type":"extras","title":"Documentation - ExCurl","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#documentation"},{"type":"extras","title":"Acknowledgements - ExCurl","doc":"This library is built on top of [libcurl](https://curl.se/libcurl/) as a NIF using [Zigler](https://github.com/ityonemo/zigler) and [Zig's C interoperability](https://ziglang.org/learn/samples/#using-curl-from-zig).\n\nInspired by [Tesla](https://github.com/elixir-tesla/tesla), [Katipo](https://github.com/puzza007/katipo), and [Req](https://github.com/wojtekmach/req).","ref":"readme.html#acknowledgements"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2022 Daniel Rudnitski\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}